//=============================================================================
// GunlocActor.
//=============================================================================
class PathsActor expands XC_Engine_Actor;

var GunlocScout Scout;

native(641) static final function bool Array_Insert_Obj( out array<Object> Ar, int Offset, optional int Count );

function SimpleConnect( NavigationPoint Other, optional int ReserveOut, optional int ReserveInc)
{
	local NavigationPoint N;
	local LiftExit Le;
	local array<NavigationPoint> Navs;
	local int i, j, iNavs, IAdds;
	local name URLName;
	local bool bConnectOne;

	//List by distance
	LockToNavigationChain( Other, true);
//	log ("Simple add to paths points...");
	if (Other.IsA('NullNode'))
	{
		ForEach NavigationActors( class'NavigationPoint', N, 2000, Other.Location, true)
		if ( N != Other )
		{
			For ( i=0 ; i<iNavs ; i++ )
				if ( VSize(N.Location - Other.Location) < VSize(Navs[i].Location - Other.Location) )
					break;
				if ( Array_Insert_Obj( Navs, i) )
				{
					Navs[i] = N;
					iNavs++;
				}
		}
	}
	//Connect this Node
//	log ("Connector...");

	if ( (Teleporter(Other) != None) && (Teleporter(Other).URL != "") )
	{
		URLName = Class'XC_CoreStatics'.static.StringToName(Teleporter(Other).URL);
		if ( URLName != '' )
			ForEach AllActors( class'NavigationPoint', N, URLName)
				if ( N.IsA('Teleporter') )
				{
					EzConnectNavigationPoints( Other, N,, true);
					ReserveOut++;
				}
	}
// Added NS
	if ( Scout == None || Scout.bDeleteMe )
		Scout = Spawn(class'GunlocScout');

	ForEach RadiusActors( class'NavigationPoint', N, 750, Other.Location)
	{
		if ( !N.IsA('LiftCenter') && N != Other && DirectLine(Other,N) )
		{
//			log ("Connect "$Other.Name$" with "$N.Name);
			if ( ( Scout.CheckReachability( Other,N ) && GunlocConnectNavigationPoints( Other, N, Scout ))
			|| ( Other.IsA('FSmartNode') && VSize(Other.Location - N.Location) < 300 ) )
			{
				EzConnectNavigationPoints( Other,N,,True );
				ReserveOut++;
			}
			if ( Scout.CheckReachability( N,Other ) && GunlocConnectNavigationPoints( N, Other, Scout )
			|| ( Other.IsA('FSmartNode') && VSize(Other.Location - N.Location) < 300 ))
			{
				EzConnectNavigationPoints( N,Other,,True );
				ReserveOut++;
			}
		}
	}

	if ( E_LiftCenter(Other) != None )
	{
		foreach AllActors (class 'LiftExit',Le)
		{
			if ( E_LiftCenter(Other).LiftTag == Le.LiftTag )
			{
				log (Other.Name$" found "$Le.Name$", operating connections...");
				EzConnectNavigationPoints( Other,Le,,True );
				ReserveOut++;
				EzConnectNavigationPoints( Le,Other,,True );
				ReserveOut++;
			}
		}
	}

	if ( Scout == None || Scout.bDeleteMe )
		Scout = Spawn(class'GunlocScout');

	//Connect outgoing paths
	For ( j=0 ; (j<16) && (Other.Paths[j] != -1) ; j++ ) {}
	For ( i=0 ; i<iNavs ; i++ )
	{
		if ( j >= 16-ReserveOut )
			break;
		if ( Scout.CheckReachability(Other,Navs[i]) && GunlocConnectNavigationPoints( Other, Navs[i],Scout ) )
		{
			BroadcastMessage(Other.Name$" is connected with "$Navs[i].Name );
			log("Scout Test Connect "$Other.Name$" and "$Navs[i].Name);
			j++;
		}
	}
	//Connect incoming paths
	For ( j=0 ; (j<16) && (Other.upstreamPaths[j] != -1) ; j++ ) {}
	For ( i=0 ; i<iNavs ; i++ )
	{
		if ( j >= 16-ReserveInc )
			break;
		if ( Scout.CheckReachability(Navs[i],Other) && GunlocConnectNavigationPoints( Navs[i],Other,Scout ) )
		{
			BroadcastMessage(Navs[i].Name$" is connected with "$Other.Name );
			log("Scout Test Connect "$Navs[i].Name$" and "$Other.Name);
			j++;
		}
	}
	CompactPathList( Other); //Also cleans up invalid paths (Start or End = NONE)
}

function RegisterPoint(NavigationPoint Other)
{
	local NavigationPoint N;
	local array<NavigationPoint> Navs;
	local int i, j, iNavs, IAdds;

	LockToNavigationChain( Other, true);
	log ("Registering"@Other.Name);
	if (Other.IsA('NullNode'))
	{
		ForEach NavigationActors( class'NavigationPoint', N, 2000, Other.Location, true)
		if ( N != Other )
		{
			For ( i=0 ; i<iNavs ; i++ )
				if ( VSize(N.Location - Other.Location) < VSize(Navs[i].Location - Other.Location) )
				{
					log("Break register"@Other.name);
					break;
				}
				if ( Array_Insert_Obj( Navs, i) )
				{
					Navs[i] = N;
					iNavs++;
					log ("Inserted"@Other.Name);
				}
		}
	}
}

final function bool DirectLine(Actor Other, Actor N)
{
	local bool result;
	local actor HitActor;
	local vector HitLocation, HitNormal;

	result=False;
	if ( Other == None || N == None )
		result=False;
	HitActor = Trace( HitLocation,HitNormal,Other.Location,N.Location,True);
	if ( HitActor == None || HitActor.bIsPawn || HitActor.IsA('Mover') )
		result = True;
	return result;
}

//EZ quick connect between both nodes
function bool GunlocConnectNavigationPoints( NavigationPoint Start, NavigationPoint End, Pawn Reference)
{
	local ReachSpec R;
	local Actor A;
	local int rIdx, pIdx;

	ForEach ConnectedDests ( Start, A, rIdx, pIdx)
		if ( A == End )
			return false;

	R.Start = Start;
	R.End = End;
	R.ReachFlags = R_WALK | R_JUMP;
	if ( Start.Region.Zone.bWaterZone || End.Region.Zone.bWaterZone )
		R.ReachFlags = R.ReachFlags | R_SWIM;
	if ( Start.IsA('LiftCenter') || End.IsA('LiftCenter') )			R.Distance = 500;
	else if ( Start.IsA('Teleporter') && End.IsA('Teleporter') )	R.Distance = 0;
	else if ( (R.ReachFlags & R_SWIM) != 0 )						R.Distance = VSize( Start.Location - End.Location) * 2;
	else															R.Distance = VSize( Start.Location - End.Location);
	R.CollisionHeight = Reference.CollisionHeight;
	R.CollisionRadius = Reference.CollisionRadius;
	AddReachSpec( R, true); //Auto-register in path
	return true;
}

function CleanupNavSpecs( NavigationPoint N)
{
	local ReachSpec R;
	local int i, RI;
	local NavigationPoint NC[2];

	ForEach AllReachSpecs( R, RI)
	{
		if ( R.Start == N || R.End == N )
		{
			NC[0] = NavigationPoint(R.Start);
			NC[1] = NavigationPoint(R.End);
			R.Start = None;
			R.End = None;
			R.bPruned = 0;
			R.Distance = 0;
			R.CollisionHeight = 0;
			R.CollisionRadius = 0;
			SetReachSpec( R, RI);
			CompactPathList(NC[0]);
			CompactPathList(NC[1]);
		}
	}
}

defaultproperties
{
	bGameRelevant=True
}
